--*****************************************************************************
-- Datei            : abwesenheitsmarkierung.adb
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 1
-- Autor            : Peter Vollmer
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 04.01.96
-------------------------------------------------------------------------------
-- Aufgabe          : Stellt Funktionen fuer den Umgang mit der Abwesenheits-
--		      markierung zur Verfuegung.	
-- Anwendung        :
-- Einschraenkung   :
-- Fehlerbehandlung :
--*****************************************************************************
with Text_Io;use Text_Io;

with Terminverwaltung;
with Benutzer;

package body Abwesenheitsmarkierung is

package Abwesenheit_File_IO is new Sequential_IO (Abwesenheitsarray_Typ);
A_Liste :  Abwesenheitsarray_Typ;
A_File :  Abwesenheit_File_IO.File_Type;
A_Liste_Name :  constant String := "Abwesenheitsmarkierung";

-------------------------------------------------------------------------------
procedure Init_A_Liste is 
-- Vorbedingung     : keine
--
-- Nachbedingung    : Die Datei "Abwesenheitsmarkierung" wird ,wenn vorhanden,
--                    gelesen.
--
-- Fehlerbehandlung : keine
-------------------------------------------------------------------------------
begin
for I in Jahr_Typ'Range loop
   for J in Monat_Typ'Range loop
      for K in Tag_Typ'Range loop
	 A_Liste(I,J,K) := Keine;
      end loop;
   end loop;
end loop;
exception  
   when others => 
      Put_Line("Fehler in Abwesenheitsmarkierung.Init_A_Liste .");
      raise;
end Init_A_liste;

-------------------------------------------------------------------------------
function Ist_Abwesend(Datum : Datum_Typ) return Boolean is
-- Vorbedingung     : Datum ist ein gueltiges Datum
--
-- Nachbedingung    : Ist der Abwesenheitsgrund 'keine' wird false , sonst
--                    true ausgegeben.
-- Fehlerbehandlung : Exception "Ungueltiges_Datum"
-------------------------------------------------------------------------------
begin
if A_Liste(Datum.Jahr,Datum.Monat,Datum.Tag) = keine then
   return False;
else
   return True;
end if;
exception  
   when others => 
      Put_Line("Fehler in Abwesenheitsmarkierung.Ist_Abwesend .");
      Put_Line ("-- Datum : " & Datum_In_String(Datum));
      raise;
end Ist_Abwesend; 

-------------------------------------------------------------------------------
function Abwesenheitsgrund(Datum : Datum_Typ) return Abwesenheits_Typ is
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum
--
-- Nachbedingung    : Liefert den Grund der Abwesenheit.
--                    
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum"
-------------------------------------------------------------------------------
begin
return A_Liste(Datum.Jahr,Datum.Monat,Datum.Tag);
exception  
   when others => 
      Put_Line("Fehler in Abwesenheitsmarkierung.Abwesenheitsgrund .");
      Put_Line ("-- Datum : " & Datum_In_String(Datum));
      raise;
end;

-------------------------------------------------------------------------------
procedure Lese_Daten is 
-- Vorbedingung     : keine
--
-- Nachbedingung    : Die Datei "Abwesenheitsmarkierung" wird ,wenn vorhanden,
--                    gelesen.
--
-- Fehlerbehandlung : keine
-------------------------------------------------------------------------------
begin
Abwesenheit_File_Io.Open(A_File,Abwesenheit_File_Io.In_File,
			 Benutzer.Pfad&A_Liste_Name);
Abwesenheit_File_Io.Read(A_File,A_Liste);
Abwesenheit_File_Io.Close(A_File);
exception  
   when Abwesenheit_File_Io.Name_Error =>
      Init_A_Liste;
   when others => 
      Put_Line("Fehler in Abwesenheitsmarkierung.Lese_Daten .");
      raise;
end Lese_Daten;

-------------------------------------------------------------------------------
procedure Schreibe_Daten is
-- Vorbedingung     : keine
--
-- Nachbedingung    : Die Datei "Abwesenheitsmarkierung" wird ,wenn 
--                    Markierungenn vorhanden geschrieben.
--
-- Fehlerbehandlung : keine
-------------------------------------------------------------------------------
begin
Abwesenheit_File_Io.Create(A_File,Abwesenheit_File_Io.Out_File,
				 Benutzer.Pfad&A_Liste_Name);
Abwesenheit_File_Io.Write(A_File,A_Liste);
Abwesenheit_File_Io.Close(A_File);
exception  
   when others => 
      Put_Line("Fehler in Abwesenheitsmarkierung.Schreibe_Daten .");
      raise;
end Schreibe_Daten;

-------------------------------------------------------------------------------
procedure Setze_Markierung(Datum1,Datum2 : in Datum_Typ;
			   Grund : in Abwesenheits_Typ) is
-- Vorbedingung     : 1) Gueltiges Datum
--
-- Nachbedingung    : Die Datei "Abwesenheitsmarkierung" wird ,wenn 
--                    Markierungenn vorhanden geschrieben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum"
-------------------------------------------------------------------------------
D1,D2,JGrenze : Kalender.Nummer_Des_Tages_Im_Jahr_typ;
Dummy : Kalender.Datum_typ;

begin
if not Ist_Groesser(Datum1, Datum2) then
   D1 := Datum_In_Nummer_Des_Tages_Im_Jahr(Datum1);
   D2 := Datum_In_Nummer_Des_Tages_Im_Jahr(Datum2);
   if Datum1.Jahr = Datum2.Jahr then
      for I in D1..D2 loop
	 Dummy := Nummer_Des_Tages_Im_Jahr_In_Datum(I,Datum1.Jahr);
	 A_Liste(Dummy.Jahr,Dummy.Monat,Dummy.Tag) := Grund;
      end loop;
   else
      if Kalender.Ist_Schaltjahr(Datum1.Jahr) then JGrenze := 366;
      else JGrenze := 365;
      end if;
      
      for I in D1..JGrenze loop
	 Dummy := Kalender.Nummer_Des_Tages_Im_Jahr_In_Datum(I,Datum1.jahr);
	 A_Liste(Dummy.Jahr,Dummy.Monat,Dummy.Tag) := Grund;
      end loop;
      for I in 1..D2 loop
	 Dummy := Kalender.Nummer_Des_Tages_Im_Jahr_In_Datum(I,Datum2.jahr);
	 A_Liste(Dummy.Jahr,Dummy.Monat,Dummy.Tag) := Grund;
      end loop;
   end if;
end if;
Schreibe_Daten;
exception  
   when others => 
      Put_Line("Fehler in Abwesenheitsmarkierung.Setz_Markierung .");
      Put_Line("-- von : "& Datum_In_String(Datum1));
      Put_Line("-- bis : "& Datum_In_String(Datum2));
      Put_Line("-- auf : "& Abwesenheits_Typ'Image(Grund));
      raise;
end;


end Abwesenheitsmarkierung;

