--*****************************************************************************
-- Datei            : ada_tcl.adb
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 
-- Autor            : 
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 
-------------------------------------------------------------------------------
-- Aufgabe          : Prozeduren fuer Tcl zur verfuegung stellen
-- Anwendung        :
-- Einschraenkung   :
-- Fehlerbehandlung :
--*****************************************************************************

with text_io; use text_io;
with sysdep; use sysdep;
with cstrings; use cstrings;
with adatcl; use adatcl;
with Stringzuweisung; use Stringzuweisung;

with Uhrzeit;
with Terminverwaltung;
with Kalender;
with Abwesenheitsmarkierung;
with Benutzer;

package body Ada_Tcl is

TRENNZEICHEN : constant String := "@";
OK_STRING : constant String := "0" & TRENNZEICHEN & TRENNZEICHEN;

--*****************************************************************************
function Ada_Markierung_Setzen (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is

Markierung : Abwesenheitsmarkierung.Abwesenheits_Typ;

Eingabe : String(1..30);
Anfang : String(1..10);
Ende : String(1..10);
A_Datum,E_Datum : Kalender.Datum_Typ;
Offset,Offset2 : Integer;

begin
Set_String( Eingabe, c2string(argi(argc, argv, 1)));
Set_String(Anfang," ");
Set_String(Ende," ");

Offset :=1;
while Eingabe(Offset)/='@' loop
   Anfang(Offset):=Eingabe(Offset);
   Offset:=Offset+1;
end loop;
Offset2:=1;
while Eingabe(Offset+Offset2)/='@' loop
   Ende(Offset2):=Eingabe(Offset+Offset2);
   Offset2:=Offset2+1;
end loop;

Block:
begin
   A_Datum:=Kalender.String_In_Datum(Anfang);
   E_Datum:=Kalender.String_In_Datum(Ende);
   if Kalender.Ist_Groesser(A_Datum,E_Datum) then
      tcl_setresult (interp, "1@Ungueltiges Intervall eingegeben.");
      return Tcl_Ok;
   end if;
exception
   when Kalender.Ungueltiges_Datum =>
      tcl_setresult (interp, "1@Ungueltiges Datum eingegeben.");
      return Tcl_Ok;
   when others => raise;
end Block;

case Eingabe(Offset+Offset2+1) is
  when '1' => Markierung := Abwesenheitsmarkierung.Urlaub;
  when '2' => Markierung := Abwesenheitsmarkierung.Tagung;
  when '3' => Markierung := Abwesenheitsmarkierung.Dienstreise;
  when others => Markierung := Abwesenheitsmarkierung.Keine;
end case;
Abwesenheitsmarkierung.Setze_Markierung(A_Datum,E_Datum,Markierung);

tcl_setresult (Interp, OK_STRING);
return Tcl_Ok;
end Ada_Markierung_Setzen;



--*****************************************************************************
function Ada_Login (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is

Eingabe : String(1..20);

begin

Set_String( Eingabe, c2string(argi(argc, argv, 1)));
Benutzer.Login_Name(Trimm_String(Eingabe));
Terminverwaltung.Lese_Daten;
Abwesenheitsmarkierung.Lese_Daten;

tcl_setresult (interp, OK_STRING);
return Tcl_Ok;
end Ada_Login;


--*****************************************************************************
function Ada_Monatsuebersicht (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is

MONATE : constant Integer :=3 ;
WOCHEN : constant Integer :=6 ;


Monat_String : String(1..2);
Monat : array (1..MONATE) of Kalender.Monat_Typ;
Jahr_String : String(1..4);
Jahr_Offset : Integer;
Jahr : array (1..MONATE) of Kalender.Jahr_Typ;

Hilfs_Datum : Kalender.Datum_Typ;
Hilfs_Zaehler : Integer;

Eingabe : String (1..10);

Datum    : array (1..MONATE,1..WOCHEN,Kalender.Wochentag_Typ'Range) of Integer;
Feiertag : array (1..MONATE,1..WOCHEN,Kalender.Wochentag_Typ'Range) of Integer;
Termin   : array (1..MONATE,1..WOCHEN,Kalender.Wochentag_Typ'Range) of Integer;
KW       : array (1..MONATE,1..WOCHEN) of Integer;

Monat_Anzeigen : array (1..MONATE) of Boolean;


begin


-------------------------------------------------------------------------------
-- Arrays initialisieren
-------------------------------------------------------------------------------

for I in 1..MONATE loop
   for J in 1..WOCHEN loop
      for K in Kalender.Wochentag_Typ'Range loop
	 Datum(I,J,K):=0;
	 Feiertag(I,J,K):=0;
	 Termin(I,J,K):=0;
      end loop;
      KW(I,J):=0;
      Feiertag(I,J,Kalender.Samstag):=1;
      Feiertag(I,J,Kalender.Sonntag):=1;
   end loop;
   Monat_Anzeigen(I):=TRUE;
end loop;

-------------------------------------------------------------------------------
-- String von Tcl auswerten und Monate/Jahre fuer Uebersicht setzen
-------------------------------------------------------------------------------
Set_String( Eingabe, c2string(argi(argc, argv, 1)));
Monat_String:="  ";
Jahr_Offset := 1;
loop
   Monat_String(Jahr_Offset) := Eingabe(Jahr_Offset);
   Jahr_Offset :=Jahr_Offset+1;
   exit when (Eingabe(Jahr_Offset) = '.');
end loop;
   for I in 1..4 loop
      Jahr_String(I) := Eingabe(I+Jahr_Offset);
   end loop;
   Monat(2) := Kalender.Monat_Typ'Value(Monat_String);
   Jahr(2) := Kalender.Jahr_Typ'Value(Jahr_String);
   
   if Kalender."="(Monat(2),Kalender.Monat_Typ'First) then
      if Kalender."="(Jahr(2),Kalender.Jahr_Typ'First) then
	 Monat_Anzeigen(1):=FALSE;
      else
	 Monat(1):=Kalender.Monat_Typ'Last;
	 Jahr(1):=Kalender."-"(Jahr(2),1);
      end if;
   else
      Monat(1):=Kalender."-"(Monat(2),1);
      Jahr(1):=Jahr(2);
   end if;
   if Kalender."="(Monat(2),Kalender.Monat_Typ'Last) then
      if Kalender."="(Jahr(2),Kalender.Jahr_Typ'Last) then
	 Monat_Anzeigen(3):=FALSE;
      else
	 Monat(3):=Kalender.Monat_Typ'First;
	 Jahr(3):=Kalender."+"(Jahr(2),1);
      end if;
   else
      Monat(3):=Kalender."+"(Monat(2),1);
      Jahr(3):=Jahr(2);
   end if;

-------------------------------------------------------------------------------
-- Arrays mit richtigen Daten fuellen
-------------------------------------------------------------------------------
for I in 1..MONATE loop
   if Monat_Anzeigen(I) then
      -- den 1. des Monats eintragen
      Hilfs_Datum := (Tag => 1, Monat => Monat(I), Jahr => Jahr(I));
      Datum(I,1,Kalender.Datum_In_Wochentag(Hilfs_Datum)):=1;
      -- die uebrigen Tage des Monats eintragen;
      Datum_Setzen:
      declare
	 Im_Monat : Boolean;
	 Aktuelles_Datum : Integer;
	 Letztes_Datum : Integer;
      begin
	 Im_Monat := FALSE;
	 Aktuelles_Datum:=1;
	 Letztes_Datum:=Integer(Kalender.Gib_Letzten_Tag_Im_Monat(Monat(I),Jahr(I)));
	 for J in 1..WOCHEN loop
	   for K  in Kalender.Wochentag_Typ'Range loop
	      if Im_Monat then
		 Aktuelles_Datum:=Aktuelles_Datum+1;
		 if Aktuelles_Datum<=Letztes_Datum then
		    Datum(I,J,K):=Aktuelles_Datum;
		 end if;
	      else
		 if Datum(I,J,K)=1 then
		    Im_Monat := TRUE;
		 end if;
	      end if;
	   end loop;
	 end loop;
      end Datum_Setzen;
      -- KW der ersten Woche des Monats eintragen
      KW (I,1):=Integer(Kalender.Datum_In_Wochennummer(Hilfs_Datum));
      -- die uebrigen KW eintragen
      if (Kalender."="(Monat(I),Kalender.Monat_Typ'First) and KW(I,1)>1) then
	 KW(I,2):=1;
	 for J in 3..WOCHEN loop
	    if Datum(I,J,Kalender.Montag)>0 then
	       KW(I,J):=KW(I,J-1)+1;
	    end if;
	 end loop;
      else
	 for J in 2..WOCHEN loop
	    if Datum(I,J,Kalender.Montag)>0 then
	       KW(I,J):=KW(I,J-1)+1;
	    end if;
	 end loop;
      end if;
      if Kalender."="(Monat(I),Kalender.Monat_Typ'Last) then
	 if KW(I,6)=53 then
	    KW(I,6):=1;
	 elsif Datum(I,5,Kalender.Wochentag_Typ'First)>28 then
	    KW(I,5):=1;
	 end if;
      end if;
      -- Feiertage und Termine eintragen
      for J in 1..WOCHEN loop
	 for K  in Kalender.Wochentag_Typ'Range loop
	    if Datum(I,J,K)>0 then
	       Hilfs_Datum.Tag:=Kalender.Tag_Typ(Datum(I,J,K));
               Hilfs_Datum.Monat := Monat(I);
	       Hilfs_Datum.Jahr := Jahr(I);
	       if Kalender.Ist_Feiertag(Hilfs_Datum) then
		  Feiertag(I,J,K):=1;
	       end if;
	       if Abwesenheitsmarkierung.Ist_Abwesend(Hilfs_Datum) then
		  Termin(I,J,K):=1;
	       end if;
	       if Terminverwaltung.Ist_Termin_Am_Tag(Hilfs_Datum) then
		  Termin(I,J,K):=2;
	       end if;
	    end if;
	 end loop;
      end loop;
   end if;
end loop;

-------------------------------------------------------------------------------
-- Ergebnis-String aus den Daten aufbauen
-------------------------------------------------------------------------------
Tcl_Setresult (Interp,OK_STRING);
for I in 1..MONATE loop
   if Monat_Anzeigen(I) then
      Tcl_Appendresult (Interp,
			Int_To_String(Integer(Monat(I))) & TRENNZEICHEN &
			Int_To_String(Integer(Jahr(I))) & TRENNZEICHEN );
   else 
      Tcl_Appendresult (Interp, "0"&TRENNZEICHEN&TRENNZEICHEN );
   end if;
end loop;
for I in 1..MONATE loop
   for J in 1..WOCHEN loop
      Tcl_Appendresult (Interp,
			Int_To_String(KW(I,J)) & TRENNZEICHEN);
   end loop;
   for K in Kalender.Wochentag_Typ'Range loop
      for J in 1..WOCHEN loop
	 Tcl_Appendresult (Interp,
			   Int_To_String(Datum(I,J,K)) & TRENNZEICHEN &
			   Int_To_String(Feiertag(I,J,K)) & TRENNZEICHEN &
			   Int_To_String(Termin(I,J,K)) & TRENNZEICHEN );
      end loop;
   end loop;
end loop;
return TCL_OK;
end Ada_Monatsuebersicht;

--*****************************************************************************
function Ada_Wochenuebersicht (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is



Eingabe : String(1..10);
Woche_String : String(1..2);
Woche : array(1..3) of Kalender.Wochennummer_Typ;
Jahr_String : String(1..4);
Jahr : array(1..3) of Kalender.Jahr_Typ;
Jahr_Offset : Integer;

Datum : Kalender.Datum_Typ;
Termin_Liste : Terminverwaltung.P_Termin_Liste_Typ;

begin

Set_String( Eingabe, c2string(argi(argc, argv, 1)));

Woche_String:="  ";
Jahr_Offset := 1;
loop
   Woche_String(Jahr_Offset) := Eingabe(Jahr_Offset);
   Jahr_Offset :=Jahr_Offset+1;
   exit when (Eingabe(Jahr_Offset) = '.');
end loop;
for I in 1..4 loop
   Jahr_String(I) := Eingabe(I+Jahr_Offset);
end loop;

Woche(2) := Kalender.Wochennummer_Typ'Value(Woche_String);
Jahr(2) := Kalender.Jahr_Typ'Value(Jahr_String);

Datum := Kalender.Erster_Tag_Der_Woche(Woche(2),Jahr(2));

KW1 :
declare
  Hilfs_Datum : Kalender.Datum_Typ;
begin
   Hilfs_Datum:=Kalender.Gib_Vorherigen_Tag(Datum);
   Woche(1):=Kalender.Datum_In_Wochennummer(Hilfs_Datum);
   Jahr(1):=Jahr(2);
exception
   when Kalender.Ungueltiges_Datum =>
      Woche(1):=Woche(2);
      Jahr(1):=Jahr(2);
   when others =>
      Put_Line("oops...kw1");
      raise;
end KW1;


KW3 :
declare
  Hilfs_Datum : Kalender.Datum_Typ;
begin
   Hilfs_Datum:=Datum;
   for I in 1..7 loop
      Hilfs_Datum:=Kalender.Gib_Naechsten_Tag(Hilfs_Datum);
   end loop;
   Woche(3):=Kalender.Datum_In_Wochennummer(Hilfs_Datum);
   Jahr(3):=Jahr(2);
exception
   when Kalender.Ungueltiges_Datum =>
      Woche(3):=Woche(2);
      Jahr(3):=Jahr(2);
   when others =>
      Put_Line("oops...kw3");
      raise;
end KW3;


if Kalender.">"(Woche(1),Woche(2)) then
   if Kalender."="(Jahr(1),Kalender.Jahr_Typ'First) then
      Woche(1):=Woche(2);
   else
      Jahr(1):=Kalender."-"(Jahr(1),1);
   end if;
end if;

if Kalender."<"(Woche(3),Woche(2)) then
   if Kalender."="(Jahr(1),Kalender.Jahr_Typ'Last) then
      Woche(3):=Woche(2);
   else
      Jahr(3) := Kalender."+"(Jahr(3),1);
   end if;
end if;

Tcl_Setresult (Interp, OK_STRING);
for I in 1..3 loop
   Tcl_Appendresult(Interp, Int_To_String(Integer(Woche(I))) & TRENNZEICHEN);
   Tcl_Appendresult(Interp, Int_To_String(Integer(Jahr(I))) & TRENNZEICHEN);
end loop;

Datum := Kalender.Erster_Tag_Der_Woche(Woche(2),Jahr(2));
Tcl_Appendresult (Interp, Kalender.Datum_In_String(Datum)&" bis ");
Block :
declare
   Hilfs_Datum : Kalender.Datum_Typ;
begin
   Hilfs_Datum := Datum;
   for I in 1..6 loop
      Hilfs_Datum := Kalender.Gib_Naechsten_Tag(Hilfs_Datum);
   end loop;
   Tcl_Appendresult (Interp,
		     Kalender.Datum_In_String(Hilfs_Datum)&TRENNZEICHEN);
exception
   when Kalender.Ungueltiges_Datum =>
      Tcl_Appendresult (Interp,
			Kalender.Datum_In_String(Hilfs_Datum)&TRENNZEICHEN);
end Block;

Block2:
begin
for I in 1..7 loop
   Tcl_Appendresult(Interp,Kalender.Wochentag_Typ'Image(
		    Kalender.Datum_In_Wochentag(Datum)) & ", " );
   Tcl_Appendresult(Interp, Kalender.Datum_In_String (Datum));
   case Abwesenheitsmarkierung.Abwesenheitsgrund(Datum) is
      when Abwesenheitsmarkierung.Keine =>
	 null;
      when Abwesenheitsmarkierung.Urlaub =>
	 Tcl_Appendresult(Interp," (Urlaub)");
      when Abwesenheitsmarkierung.Tagung =>
	 Tcl_Appendresult(Interp," (Tagung)");
      when Abwesenheitsmarkierung.Dienstreise =>
	 Tcl_Appendresult(Interp," (Dienstreise)");
   end case;
   Tcl_Appendresult(Interp,TRENNZEICHEN);

   Termin_Liste :=  Terminverwaltung.Gib_Termine_Des_Tages(Datum);
   while Terminverwaltung."/="(Termin_Liste, null) loop
      Tcl_Appendresult(Interp , "  ");
      Tcl_Appendresult(Interp ,
		    Uhrzeit.Zeit_In_String(Termin_Liste.Termin.Anfangszeit));
      Tcl_Appendresult(Interp , " - ");
      Tcl_Appendresult(Interp ,
	            Uhrzeit.Zeit_In_String(Termin_Liste.Termin.Endzeit));
      Tcl_Appendresult(Interp , " ");
      Tcl_Appendresult(Interp ,
		       Trimm_String(String(Termin_Liste.Termin.Kurzbeschreibung)));
      Tcl_Appendresult(Interp , TRENNZEICHEN);
      
      Termin_Liste := Termin_Liste.Naechster;
   end loop;
Datum := Kalender.Gib_Naechsten_Tag(Datum);
end loop;
exception
   when Kalender.Ungueltiges_Datum =>
      null;
end Block2;
  
  return Tcl_Ok;
end Ada_Wochenuebersicht;

--*****************************************************************************
function Ada_Tagesuebersicht (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is

Eingabe : String(1..10);
Datum : array(1..3) of Kalender.Datum_Typ;
Termin_Liste : Terminverwaltung.P_Termin_Liste_Typ;

begin

Set_String( Eingabe, c2string(argi(argc, argv, 1)));

Datum(2):=Kalender.String_In_Datum(Eingabe);

Vorheriger_Tag :
begin
   Datum(1):=Kalender.Gib_Vorherigen_Tag(Datum(2));
exception
   when Kalender.Ungueltiges_Datum => Datum(1):=Datum(2);
end Vorheriger_Tag;

Naechster_Tag :
begin
   Datum(3):=Kalender.Gib_Naechsten_Tag(Datum(2));
exception
   when Kalender.Ungueltiges_Datum => Datum(3):=Datum(2);
end Naechster_Tag;

Tcl_Setresult (Interp, OK_STRING);
for I in 1..3 loop
   Tcl_Appendresult(Interp, Kalender.Datum_In_String (Datum(I))&TRENNZEICHEN);
end loop;
Tcl_Appendresult(Interp,Kalender.Wochentag_Typ'Image(Kalender.Datum_In_Wochentag(Datum(2))) & TRENNZEICHEN );
Tcl_Appendresult(Interp , Trimm_String(Kalender.Feiertagname(Datum(2)))
		          & TRENNZEICHEN );
case Abwesenheitsmarkierung.Abwesenheitsgrund(Datum(2)) is
   when Abwesenheitsmarkierung.Keine =>
      null;
   when Abwesenheitsmarkierung.Urlaub =>
      Tcl_Appendresult(Interp,"Urlaub");
   when Abwesenheitsmarkierung.Tagung =>
      Tcl_Appendresult(Interp,"Tagung");
   when Abwesenheitsmarkierung.Dienstreise =>
      Tcl_Appendresult(Interp,"Dienstreise");
end case;
Tcl_Appendresult(Interp , TRENNZEICHEN );

Termin_Liste :=  Terminverwaltung.Gib_Termine_Des_Tages(Datum(2));
while Terminverwaltung."/="(Termin_Liste, null) loop
   Tcl_Appendresult(Interp , "  ");
   Tcl_Appendresult(Interp ,
		    Uhrzeit.Zeit_In_String(Termin_Liste.Termin.Anfangszeit));
   Tcl_Appendresult(Interp , " - ");
   Tcl_Appendresult(Interp ,
		    Uhrzeit.Zeit_In_String(Termin_Liste.Termin.Endzeit));
   Tcl_Appendresult(Interp , " ");
   Tcl_Appendresult(Interp ,
	       Trimm_String(String(Termin_Liste.Termin.Kurzbeschreibung)));
   Tcl_Appendresult(Interp , TRENNZEICHEN);

   Termin_Liste := Termin_Liste.Naechster;
end loop;
return Tcl_Ok;
end Ada_Tagesuebersicht;

--*****************************************************************************
function Ada_Termindaten (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is

Eingabe : String(1..20);
Datum_String : String(1..10);
Datum : Kalender.Datum_Typ;
Zeit_String : String(1..5);
Zeit : Uhrzeit.Uhrzeit_Typ;
Offset : Integer;

Termin : Terminverwaltung.Termin_Typ;
Teilnehmer2 : Terminverwaltung.P_Teilnehmer_Liste_Typ;

begin

Set_String( Eingabe, c2string(argi(argc, argv, 1)));
Offset:=1;
Set_String (Datum_String," ");
while Eingabe(Offset)/='@' loop
   Datum_String(Offset):=Eingabe(Offset);
   Offset:=Offset+1;
end loop;

Zeit_String:=Eingabe(Offset+1..Offset+5);

Datum := Kalender.String_In_Datum(Datum_String);
Zeit := Uhrzeit.String_In_Zeit(Zeit_String);

Termin := Terminverwaltung.Gib_Termin(Zeit,Datum);

Tcl_Setresult (Interp, OK_STRING);
Tcl_Appendresult(Interp , Kalender.Datum_In_String(Termin.Datum));
Tcl_Appendresult(Interp , TRENNZEICHEN);
Tcl_Appendresult(Interp , Uhrzeit.Zeit_In_String(Termin.Anfangszeit));
Tcl_Appendresult(Interp , TRENNZEICHEN);
Tcl_Appendresult(Interp , Uhrzeit.Zeit_In_String(Termin.Endzeit));
Tcl_Appendresult(Interp , TRENNZEICHEN);
Tcl_Appendresult(Interp , Trimm_String(String(Termin.Kurzbeschreibung)));
Tcl_Appendresult(Interp , TRENNZEICHEN);
Tcl_Appendresult(Interp , Trimm_String(String(Termin.Ort)));
Tcl_Appendresult(Interp , TRENNZEICHEN);
Tcl_Appendresult(Interp , Trimm_String(String(Termin.Kommentar(1))));
Tcl_Appendresult(Interp , TRENNZEICHEN);
Tcl_Appendresult(Interp , Trimm_String(String(Termin.Kommentar(2))));
Tcl_Appendresult(Interp , TRENNZEICHEN);
Tcl_Appendresult(Interp , Trimm_String(String(Termin.Kommentar(3))));
Tcl_Appendresult(Interp , TRENNZEICHEN);

Teilnehmer2 := Termin.Teilnehmer_Liste;
while Terminverwaltung."/="(Teilnehmer2,null) loop 
   Tcl_Appendresult(Interp , Trimm_String(String(Teilnehmer2.Name)));
   Tcl_Appendresult(Interp , TRENNZEICHEN);
   Teilnehmer2 := Teilnehmer2.Naechster;
end loop;

    return Tcl_Ok;
end Ada_Termindaten;

--*****************************************************************************
function Ada_Termin_Loeschen (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is

Eingabe : String(1..20);
Datum_String : String(1..10);
Datum : Kalender.Datum_Typ;
Zeit_String : String(1..5);
Zeit : Uhrzeit.Uhrzeit_Typ;
Offset : Integer;


begin

Set_String( Eingabe, c2string(argi(argc, argv, 1)));
Offset:=1;
Set_String (Datum_String," ");
while Eingabe(Offset)/='@' loop
   Datum_String(Offset):=Eingabe(Offset);
   Offset:=Offset+1;
end loop;

Zeit_String:=Eingabe(Offset+1..Offset+5);

Datum := Kalender.String_In_Datum(Datum_String);
Zeit := Uhrzeit.String_In_Zeit(Zeit_String);

Terminverwaltung.Termin_Loeschen(Zeit,Datum);

tcl_setresult (interp, OK_STRING);
return Tcl_Ok;
end Ada_Termin_Loeschen;

--*****************************************************************************
function Ada_Termin_Eintragen (cdata:  in tcl_clientdata;
                 interp: in tcl_interp;
                 argc:   in integer;
                 argv:   in tcl_cstringvec
  ) return Tcl_Status is

Eingabe : String (1..1000);
Eingabe_Offset : Integer;
Teil : String(1..100);

Termin : Terminverwaltung.Termin_Typ;

-- Datum : Kalender.Datum_Typ;
-- Anfangszeit,Endzeit : Uhrzeit.Uhrzeit_Typ;
-- Kurzbeschreibung : Terminverwaltung.Kurzbeschreibung_Typ;
-- Ort : Terminverwaltung.Ort_Typ;
-- Kommentar : Terminverwaltung.Kommentar_Typ;
Hilfs_P : Terminverwaltung.P_Teilnehmer_Liste_Typ;

procedure Scan_String (Var : in     String ;
			  Str : in out String ;
			  Pos : in out Integer ) is
      Offset : Integer;
   begin
      if Pos>=Var'Length then
	 Set_String (Str,TRENNZEICHEN);
      else
	 Set_String (Str," ");
   Offset := 0;
   while((Pos+Offset<=Var'Length) and
	 (Var(Pos+Offset+Var'First-1)/='@')) loop
      if (Str'First+Offset<=Str'Last) then
	 Str(Str'First+Offset):=Var(Pos+Offset+Var'First-1);
      end if;	 
      Offset:=Offset+1;
   end loop;
   Pos:=Pos+Offset+1;
      end if;
end Scan_String;
   


begin
Set_String( Eingabe, c2string(argi(argc, argv, 1)));
Eingabe_Offset := 1;

Datum_Scan :
begin
      Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
      Termin.Datum := Kalender.String_In_Datum(Teil);
exception
   when Kalender.Ungueltiges_Datum => 
      tcl_setresult (interp, "1@Kein gueltiges Datum eingegeben.");
      return Tcl_Ok;
end Datum_Scan;

Anfangszeit_Scan :
begin      
      Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
      Termin.Anfangszeit := Uhrzeit.String_In_Zeit(Teil);
exception
   when Uhrzeit.Ungueltige_Uhrzeit =>
      tcl_setresult (interp, "1@Ungueltige Anfangszeit eingegeben.");
      return Tcl_Ok;
end Anfangszeit_Scan;

Endzeit_Scan :
begin
      Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
      Termin.Endzeit := Uhrzeit.String_In_Zeit(Teil);
exception
   when Uhrzeit.Ungueltige_Uhrzeit =>
      tcl_setresult (interp, "1@Ungueltige Endzeit eingegeben.");
      return Tcl_Ok;
end Endzeit_Scan;

Uhrzeit_Test :
declare 
   Dauer : Uhrzeit.Dauer_Typ ;
begin
      if Uhrzeit."="(Termin.Anfangszeit,Termin.Endzeit) then
	 tcl_setresult (interp, "1@Ungueltiges Zeitintervall eingegeben.");
	 return Tcl_Ok;
      end if;
      Dauer := Uhrzeit.Zeitdifferenz(Termin.Anfangszeit,Termin.Endzeit);
exception 
   when Uhrzeit.Ungueltige_Dauer =>
      tcl_setresult (interp, "1@Ungueltigs Zeitintervall eingegeben.");
      return Tcl_Ok;
end Uhrzeit_Test;


Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
Set_String(String(Termin.Kurzbeschreibung),Teil);
if Trimm_String(String(Termin.Kurzbeschreibung))=" " then
      tcl_setresult (interp, "1@Keine Kurzbeschreibung eingegeben.");
      return Tcl_Ok;
end if;


Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
Set_String(String(Termin.Ort),Teil);

for I in 1..3 loop
   Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
   Set_String(Termin.Kommentar(I),Teil);
end loop;


Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
if (Trimm_String(Teil)/=" ") then
   Termin.Teilnehmer_Liste := new Terminverwaltung.Teilnehmer_Liste_Typ;
   Set_String(String(Termin.Teilnehmer_Liste.Name),Teil);
   Hilfs_P :=Termin.Teilnehmer_Liste;
end if;
loop
   Scan_String(Trimm_String(Eingabe),Teil,Eingabe_Offset);
   exit when (Teil(1..1)=TRENNZEICHEN);
   Hilfs_P.Naechster := new Terminverwaltung.Teilnehmer_Liste_Typ;
   Hilfs_P := Hilfs_P.Naechster;
   Set_String(String(Hilfs_P.Name),Teil);
end loop;

Termin.Ist_Periodisch:=FALSE;

if Terminverwaltung.Ist_Termin_Moeglich(Termin) then
   Terminverwaltung.Termin_Eintragen(Termin);
   tcl_setresult (interp, OK_STRING);
else
   tcl_setresult (interp, "1@Zu diesem Zeitpunkt existiert bereits ein Termin.");
end if;
return Tcl_Ok;
end Ada_Termin_Eintragen;

end Ada_Tcl;
