--*****************************************************************************
-- Datei            : kalender.adb
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 1
-- Autor            : Peter Vollmer
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 04.01.96
-------------------------------------------------------------------------------
-- Aufgabe          : Stellt Funktionen fuer den Umgang mit den Kalender-
--		      funktionen zur Verfuegung.	
-- Anwendung        :
-- Einschraenkung   :
-- Fehlerbehandlung :
--*****************************************************************************

with Stringzuweisung; use Stringzuweisung;

package body kalender is

type Feiertagsreferenz_Typ is array(Tag_Typ'range,Monat_Typ'range) of                                                                           String(1..21);

Feiertagsreferenz      : Feiertagsreferenz_Typ;
Aktuelles_Referenzjahr : Jahr_Typ;

-------------------------------------------------------------------------------
function Ist_Groesser(Datum1,Datum2 : Datum_Typ) return Boolean is
			  
-- Vorbedingung     : 1) Gueltiges Datum
--
-- Nachbedingung    : vergleich zweier Datum_typen 
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum"
-------------------------------------------------------------------------------
D1,D2 :  Nummer_Des_Tages_Im_Jahr_typ;
begin
  if Datum1.Jahr > Datum2.Jahr then return True;
  elsif  Datum1.Jahr =  Datum2.Jahr then
     D1 := Datum_In_Nummer_Des_Tages_Im_Jahr(Datum1);
     D2 := Datum_In_Nummer_Des_Tages_Im_Jahr(Datum2); 
     if D1 > D2 then return True;
     else return False;
     end if;
  else return False;
  end if;
exception
   when others =>
      Put_Line ("Fehler in Kalender.Ist_Grosser .");
      Put_Line ("-- Datum1 : "&Datum_In_String(Datum1));
      Put_Line ("-- Datum2 : "&Datum_In_String(Datum2));
end;



-------------------------------------------------------------------------------
function Ist_Schaltjahr 
  (Jahr : Jahr_Typ)
   return Boolean is 
-- Vorbedingung     : 1) Jahr ist gueltiges Jahr
--
-- Nachbedingung    : Liefert TRUE wenn Datum in einem Schaltjahr liegt sonst 
--                    FALSE.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-- Erklaerung       : Alle Jahre welche durch 4 ohne Rest teilbar sind ausser
--                    den Jahren welche durch 100 aber nicht durch 400 teilbar
--                    sind. 
-------------------------------------------------------------------------------
begin
 if ((Jahr mod 400) = 0) then return TRUE;
 elsif ((Jahr mod 100) = 0) then return FALSE;
 elsif ((Jahr mod 4) = 0) then return TRUE;
 else return FALSE;
 end if;
exception
   when others =>
      Put_Line ("Fehler in Kalender.Ist_Schaltjahr .");
      Put_Line ("-- Jahr :"&Jahr_Typ'Image(Jahr));
end Ist_Schaltjahr;
 
-------------------------------------------------------------------------------
procedure Initialisiere_Feiertagsreferenz is
-- Vorbedingung     : keine
--
-- Nachbedingung    : Feiertagsreferenz wird mit Leerstringd "" initialisiert.
--
-- Fehlerbehandlung : keine
-------------------------------------------------------------------------------

begin
  for T in Tag_Typ'range loop
    for I in Monat_Typ'range loop
      Feiertagsreferenz(T,I) := "xxxxxxxxxxxxxxxxxxxxx";
    end loop;
  end loop;
end Initialisiere_Feiertagsreferenz ;

-------------------------------------------------------------------------------
procedure Berechne_Feiertage(Jahr_In : in Jahr_Typ) is
-- Vorbedingung     : 1) Eingabe gueltiges Jahr
--
-- Nachbedingung    : Ausgefuellte Feiertagsreferenz des eingegebenen Jahres.
--
-- Fehlerbehandlung : 1) keine
-- Quelle Algorithmus :  http://phdrw3.kfk.de:8080/djg/d_holi_d.html
-------------------------------------------------------------------------------

Jahr,A,B,C,D,E,OM,OT : Integer;
Datum : Datum_Typ;
Tag_Nr_Im_Jahr : Nummer_Des_Tages_Im_Jahr_Typ;
begin

  Jahr := Integer(Jahr_In);
  Initialisiere_Feiertagsreferenz;
  Aktuelles_Referenzjahr := Jahr_In;
  --  Konstante Feiertage : --

  Feiertagsreferenz(1,1)  := "Neujahr              ";
  Feiertagsreferenz(6,1)  := "Erscheinungsfest     ";
  Feiertagsreferenz(1,5)  := "Maifeiertag          ";
  Feiertagsreferenz(3,10) := "Tag der dt. Einheit  ";
  Feiertagsreferenz(1,11) := "Allerheiligen        ";
  Feiertagsreferenz(25,12):= "1. Weihnachtsfeiertag";
  Feiertagsreferenz(26,12):= "2. Weihnachtsfeiertag";
  --  Berechnung Von Ostern --
  
    A := Jahr mod 19;
    B := Jahr mod 4;
    C := Jahr mod 7;
    D := (19 * A + 24) mod 30;
    E := (2 * B + 4 * C + 6 * D + 5) mod 7;
    OT := 22 + D + E;
    OM := 3;
    if OT > 31 then
       OT := D + E - 9;
       OM := 4;
    end if;
    if ((OT = 26) and (OM = 4)) then
       OT := 19;
    end if;
    if ((OT = 25) and (OM = 4) and (D = 28) and (E = 6) and (A > 10)) then
       OT := 18;
    end if;
  
    -- extra fuer das Jahr 2100 --
    if Jahr = 2100 then Ot := Ot + 1;
       end if;



   Feiertagsreferenz(Tag_Typ(OT),Monat_Typ(OM))  := "Ostersonntag         ";
   Datum.Jahr := Jahr_In;
   Datum.Monat := Monat_Typ(OM);
   Datum.Tag := Tag_Typ(OT);
   Tag_Nr_Im_Jahr := Datum_In_Nummer_Des_Tages_Im_Jahr(Datum);
   Tag_Nr_Im_Jahr := Nummer_Des_Tages_Im_Jahr_Typ(Integer(Tag_Nr_Im_Jahr)+1);
   Datum := Nummer_Des_Tages_Im_Jahr_In_Datum(Tag_Nr_Im_Jahr,Jahr_In);  
   Feiertagsreferenz(Datum.Tag,Datum.Monat):= "Ostermontag          ";
  
   Tag_Nr_Im_Jahr := Nummer_Des_Tages_Im_Jahr_Typ(Integer(Tag_Nr_Im_Jahr)-3);
   Datum := Nummer_Des_Tages_Im_Jahr_In_Datum(Tag_Nr_Im_Jahr,Jahr_In);
   Feiertagsreferenz(Datum.Tag,Datum.Monat):= "Karfreitag           ";

   Tag_Nr_Im_Jahr :=Nummer_Des_Tages_Im_Jahr_Typ(Integer(Tag_Nr_Im_Jahr)+2+49);
   Datum := Nummer_Des_Tages_Im_Jahr_In_Datum(Tag_Nr_Im_Jahr,Jahr_In);
   Feiertagsreferenz(Datum.Tag,Datum.Monat):= "Pfingssonntag        ";

   Tag_Nr_Im_Jahr :=Nummer_Des_Tages_Im_Jahr_Typ(Integer(Tag_Nr_Im_Jahr)+1);
   Datum := Nummer_Des_Tages_Im_Jahr_In_Datum(Tag_Nr_Im_Jahr,Jahr_In);
   Feiertagsreferenz(Datum.Tag,Datum.Monat):= "Pfingstmontag        ";

   Tag_Nr_Im_Jahr :=Nummer_Des_Tages_Im_Jahr_Typ(Integer(Tag_Nr_Im_Jahr)+10);
   Datum := Nummer_Des_Tages_Im_Jahr_In_Datum(Tag_Nr_Im_Jahr,Jahr_In);
   Feiertagsreferenz(Datum.Tag,Datum.Monat):= "Fronleichnam         ";

   Tag_Nr_Im_Jahr :=Nummer_Des_Tages_Im_Jahr_Typ(Integer(Tag_Nr_Im_Jahr)-21);
   Datum := Nummer_Des_Tages_Im_Jahr_In_Datum(Tag_Nr_Im_Jahr,Jahr_In);
   Feiertagsreferenz(Datum.Tag,Datum.Monat):= "Himmelfahrt          ";
end;
-------------------------------------------------------------------------------
function Ist_Feiertag 
  (Datum : Datum_Typ)
   return Boolean is
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Ist Datum ein Feiertag wird TRUE zurueckgegeben,
--			 sonst FALSE.
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

begin
if (Datum.Jahr /= Aktuelles_Referenzjahr) then
   Berechne_Feiertage(Datum.Jahr);
end if;
if (Feiertagsreferenz(Datum.Tag,Datum.Monat) /= "xxxxxxxxxxxxxxxxxxxxx")
then return True;
else return False;  
end if;
exception
   when others =>
      Put_Line ("Fehler in Kalender.Ist_Feiertag .");
end;


-------------------------------------------------------------------------------
function Feiertagname 
  (Datum : Datum_Typ)
   return String  is 
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Ist Datum ein Feiertag wird der Name des Feiertags 
--                    zurueckgegeben, sonst ein leerer String .
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------
begin
  if (Datum.Jahr /= Aktuelles_Referenzjahr) then
    Berechne_Feiertage(Datum.Jahr);
  end if;
  if (Feiertagsreferenz(Datum.Tag,Datum.Monat) /= "xxxxxxxxxxxxxxxxxxxxx")
     then return Feiertagsreferenz(Datum.Tag,Datum.Monat);
     else return "";  
  end if;
end;


-------------------------------------------------------------------------------
function Wochentag_In_Wochentag_Nummer 
  (Wochentag : Wochentag_Typ)
   return Wochentag_Nummer_Typ is  
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Liefert den Wochentag des Datums zurueck.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

begin
  case Wochentag is
    when Montag     => return 1;
    when Dienstag   => return 2;
    when Mittwoch   => return 3;
    when Donnerstag => return 4;
    when Freitag    => return 5;
    when Samstag    => return 6;
    when Sonntag    => return 7;
   end case; 
end Wochentag_In_Wochentag_Nummer;

-------------------------------------------------------------------------------
function Datum_In_Wochentag 
  (Datum : Datum_Typ)
   return Wochentag_Typ is  
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Liefert den Wochentag des Datums zurueck.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

Diff : Integer;

begin
  Diff := 0;
  for I in 1990..Datum.Jahr-1 loop
     if Ist_Schaltjahr(I) then Diff := Diff +2;
     else Diff := Diff +1;
     end if;
  end loop;
  Diff := Diff+Integer(Datum_In_Nummer_Des_Tages_Im_Jahr(Datum));
  Diff := Diff mod 7;

  
  if (diff < 1) then diff := diff+7;
  end if;   
  case Diff is
    when 1 => return Montag;
    when 2 => return Dienstag;
    when 3 => return Mittwoch;
    when 4 => return Donnerstag;
    when 5 => return Freitag;
    when 6 => return Samstag;
    when 7 => return Sonntag;  
    when others => return Montag;
  end case; 
  
     


end Datum_In_Wochentag;

-------------------------------------------------------------------------------
  function Datum_In_Nummer_Des_Tages_Im_Jahr 
  (Datum : Datum_Typ)
   return  Nummer_Des_Tages_Im_Jahr_Typ is
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Liefert die Nummer Des Tages im Jahr.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum"
-------------------------------------------------------------------------------
Tag_Mehr : Integer;  --- Gleicht einen Tag bei einem
                     --- Schaltjahr aus.
Tag      : Integer;                         

begin
  if Ist_Schaltjahr(Datum.Jahr) then Tag_Mehr :=1;
  else Tag_Mehr := 0;
  end if;
  Tag := Integer(Datum.Tag);
  case Datum.Monat is
    when  1 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag);
    when  2 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+31);
    when  3 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+31+28+Tag_Mehr);
    when  4 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+62+28+Tag_Mehr);
    when  5 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+92+28+Tag_Mehr);
    when  6 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+123+28+Tag_Mehr);
    when  7 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+153+28+Tag_Mehr);
    when  8 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+184+28+Tag_Mehr);
    when  9 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+215+28+Tag_Mehr);
    when 10 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+245+28+Tag_Mehr);
    when 11 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+276+28+Tag_Mehr);
    when 12 => return Nummer_Des_Tages_Im_Jahr_Typ(Tag+306+28+Tag_Mehr);
  end case; 
exception
   when others =>
      Put_Line(" exception in Datum_In_Nummer_Des_Tages_Im_Jahr");
end;

-------------------------------------------------------------------------------
function Nummer_Des_Tages_Im_Jahr_In_Datum 
  (Nummer : Nummer_Des_Tages_Im_Jahr_Typ;Jahr : Jahr_Typ)
   return Datum_Typ is
-- Vorbedingung     : 1) Nummer vom richtigen Typ.
--
-- Nachbedingung    : Liefert das Datum zur Nummer des Tages im Jahr.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige Nummer_Des_Tages_Im_Jahr)
-------------------------------------------------------------------------------
type Monatsliste_Typ is Array(Monat_Typ'range) of Tag_Typ;
Monatsliste : Monatsliste_Typ;
Datum : Datum_Typ;
Nr : Nummer_Des_Tages_Im_Jahr_Typ;
begin
  Nr := Nummer;
  Monatsliste := (31,28,31,30,31,30,31,31,30,31,30,31);
  if Ist_Schaltjahr(Jahr) then Monatsliste(2) := 29;
  end if;
    for I in Monat_Typ'Range loop
    if (Integer(Nr)) > Integer((Monatsliste(I))) then 
      Nr := Nummer_Des_Tages_Im_Jahr_Typ(Integer(Nr-Nummer_Des_Tages_Im_Jahr_Typ(Monatsliste(I))));
      else Datum.Tag   := Tag_Typ(Nr);
         Datum.Monat := Monat_Typ(I);
         Datum.Jahr  := Jahr;
         exit;
    end if;
  end loop;
  return Datum;
exception
   when others =>
      Put_Line(" exception in Nummer_Des_Tages_Im_Jahr_In_Datum");
end;

-------------------------------------------------------------------------------
function Datum_In_Wochennummer 
  (Datum : Datum_Typ)
   return Wochennummer_Typ is
-- Vorbedingung     : 1) Tag vom richtigen Typ.
--
-- Nachbedingung    : Liefert zu einem Datum die zugehoerige Wochennummer.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-- Algorithmus      : Liegt der Donnerstag der Woche im alten Jahr, faengt das neue Jahr
--                    Mit der letzten Wochennummer des alten Jahres an. Sonst liegt die 
--                    Woche mit dem 1. Januar im neuen Jahr.
-------------------------------------------------------------------------------
Help_Tag : Nummer_Des_Tages_Im_Jahr_Typ;
Wochentag_1_Januar : Wochentag_Nummer_Typ;
Wochentag_1_Januar_Naechstes_Jahr : Wochentag_Nummer_Typ;
Erster_Januar : Datum_Typ;
Letzter_Dezember : Datum_Typ;
Wochennr : Wochennummer_Typ;
Datum1 : Datum_Typ;
Datum2 : Datum_Typ;

begin
   Erster_Januar.Tag := 1;
   Erster_Januar.Monat := 1;
   Erster_Januar.Jahr := Datum.Jahr; 
   Letzter_Dezember.Tag := 31;
   Letzter_Dezember.Monat := 12;
   Letzter_Dezember.Jahr := Datum.Jahr;
   Help_Tag := Datum_In_Nummer_Des_Tages_Im_Jahr(Datum);
   Wochentag_1_Januar :=Wochentag_In_Wochentag_Nummer(Datum_In_Wochentag(Erster_Januar));
   Wochentag_1_Januar_Naechstes_Jahr :=Wochentag_In_Wochentag_Nummer(Datum_In_Wochentag(Letzter_Dezember));
   Wochentag_1_Januar_Naechstes_Jahr :=(Wochentag_1_Januar_Naechstes_Jahr mod 7)+1;
   
   if (Wochentag_1_Januar = 1) or (Wochentag_1_Januar = 2) or   (Wochentag_1_Januar = 3) or
     (Wochentag_1_Januar = 4) then
     Wochennr := Wochennummer_Typ(((Integer(Wochentag_1_Januar)+Integer(Help_Tag)+5) / 7)+1);
   else 
      Wochennr := Wochennummer_Typ((Integer(Wochentag_1_Januar)+Integer(Help_Tag)+5) / 7);
      Datum1.Tag  := 31;
      Datum1.Jahr := Datum.Jahr-1;
      Datum1.Monat := 12;
      if (Wochennr = 1) then Wochennr := Datum_In_Wochennummer(Datum1)+1;
      end if;
   end if;

   if ((Wochentag_1_Januar_Naechstes_Jahr = 2) or (Wochentag_1_Januar_Naechstes_Jahr = 3) or (Wochentag_1_Januar_Naechstes_Jahr = 4)) and (Wochennr > 50) then
      if Datum.Jahr < Jahr_Typ'Last then
	 Datum2 := Erster_Tag_Der_Woche (1,Datum.Jahr+1);
	 if Datum.Tag >= Datum2.Tag then
	    return 1;
	 end if;
      end if;   
   end if;
   return Wochennr-1;
exception
   when others => 
      Put_Line(" exception in datum_in_wn !!!  ");
      Put_Line("-- Datum : "&Datum_In_String(Datum));
      raise;
end Datum_In_Wochennummer;

-------------------------------------------------------------------------------
function Erster_Tag_Der_Woche 
  (Woche : Wochennummer_Typ;Jahr : Jahr_Typ)
   return Datum_typ is
-- Vorbedingung     : 1) Woche vom richtigen Typ.
--
-- Nachbedingung    : Liefert zur Wochennummer den ersten Tag der Woche.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige Wochennummer".
-------------------------------------------------------------------------------
Datum1 : Datum_Typ;
Help_Tag : Tag_Typ;
Wochentag_1_Januar : Wochentag_Nummer_Typ;
I : Integer;
Rueckgabe : Integer;

begin
   
   Datum1.Jahr := Jahr;
   Datum1.Monat := 1;
   Datum1.Tag := 1;
   

   Wochentag_1_Januar :=Wochentag_In_Wochentag_Nummer(Datum_In_Wochentag(Datum1));
   I := Integer(Wochentag_1_Januar);
   
   if (Woche = 1) And ((Wochentag_1_Januar=2)or(Wochentag_1_Januar=3)or(Wochentag_1_Januar=4)) then 
      Datum1.Tag := Tag_Typ(33-(Integer(Wochentag_1_Januar)));
      Datum1.Monat := 12;
      Datum1.Jahr := Jahr-1;
      return Datum1;
   end if;

   I := I-2;
   if I < 3 then I := I + 7;
   end if;

   Rueckgabe := (Integer(Woche) * 7) - I;

     
-- Help_Tag :=1;
-- return Help_Tag;

return Nummer_Des_Tages_Im_Jahr_In_Datum(Nummer_Des_Tages_Im_Jahr_Typ(Rueckgabe),Jahr);
exception 
   when others => 
      Put_Line(" exception in erster_tag_der_woche "); 
      Put_Line("-- Woche : "&Wochennummer_Typ'Image(Woche));
      Put_Line("-- Jahr  : "&Jahr_Typ'Image(Jahr));
      raise;
end;

-------------------------------------------------------------------------------
function Datum_In_String 
  (Datum : Datum_Typ)
   return String is
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum. 
--
-- Nachbedingung    : Wandelt Datum_Typ in String. 
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-- 		      2) Exception "Konvertierungsfehler".
-------------------------------------------------------------------------------

begin
   return( Int_To_String(Integer(Datum.Tag)) & "." &
	   Int_To_String(Integer(Datum.Monat)) & "." &
	   Int_To_String(Integer(Datum.Jahr)));
end;

-------------------------------------------------------------------------------
function String_In_Datum 
  (Datumstring : String)
   return Datum_Typ is
-- Vorbedingung     : 1) Datumstring ist gueltiger String.
--		     	
--
-- Nachbedingung    : Liefert das Datum aus einem String.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiger_String".
--		      2) Exception "Konvertierungsfehler".
------------------------------------------------------------------------------


Tag_String : String(1..2);
Monat_String : String(1..2);
Jahr_String : String(1..4);
Offset,Offset2 : Integer;
Datum : Datum_Typ;
begin
   Tag_String := "  ";
   Monat_String := "  ";
   Jahr_String := "    ";
   Offset := 1;
   Offset2 := 1;

   loop
      Tag_String(Offset) := Datumstring(Offset);
      Offset :=Offset+1;
      exit when (Datumstring(Offset) = '.');
   end loop;
   loop
      Monat_String(Offset2) := Datumstring(Offset+Offset2);
      Offset2 :=Offset2+1;
      exit when (Datumstring(Offset+Offset2) = '.');
   end loop;
   
   for I in 1..4 loop
      Jahr_String(I) := Datumstring(I+Offset+Offset2);
   end loop;
   
   Datum.Tag :=Tag_Typ'Value(Tag_String);
   Datum.Monat :=Monat_Typ'Value(Monat_String);
   Datum.Jahr:=Jahr_Typ'Value(Jahr_String);
   if Datum.Tag > Gib_Letzten_Tag_Im_Monat(Datum.Monat,Datum.Jahr) then
      raise Ungueltiges_Datum;
   end if;
   return Datum;
   exception when Constraint_Error => raise Ungueltiges_Datum;
end;

-------------------------------------------------------------------------------
function Gib_Letzten_Tag_Im_Monat
  (Monat : Monat_Typ;Jahr : Jahr_Typ)
   return Tag_Typ is
-- Vorbedingung     : 1) Monat vom richtigen Typ.
--
-- Nachbedingung    : Liefert zur Wochennummer den ersten Tag der Woche.
--
-- Fehlerbehandlung : 1) Exception "Ungueltig_Tagesnummer".
-------------------------------------------------------------------------------
begin

  case Monat is
    when 1|3|5|7|8|10|12 => return 31;
    when 4|6|9|11        => return 30;
    when 2               => if Ist_Schaltjahr(Jahr) then return 29;
                            else return 28;
                            end if;
  end case;

end Gib_Letzten_Tag_Im_Monat;

-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Gib_Vorherigen_Tag(Datum : Datum_Typ) return Datum_Typ is
-- Vorbedingung     : 1) Gueltiges Datum
--
-- Nachbedingung    : Liefert das Datum des vorherigen Tags. 
--
-- Fehlerbehandlung : 1) Ungueltiges Datum
-------------------------------------------------------------------------------
Datum_2 : Datum_Typ;
begin
   if (Datum.Tag=Tag_Typ'First ) then
      if (Datum.Monat=Monat_Typ'First ) then
	 if (Datum.Jahr=Jahr_Typ'First ) then
	    raise Ungueltiges_Datum;
	 else
	    Datum_2.Jahr :=Datum.Jahr-1;
	    Datum_2.Monat := Monat_Typ'Last;
	    Datum_2.Tag := Gib_Letzten_Tag_Im_Monat(Datum_2.Monat,
						    Datum_2.Jahr);
	 end if;
      else
	 Datum_2.Jahr := Datum.Jahr;
	 Datum_2.Monat := (Datum.Monat-1);
	 Datum_2.Tag := Gib_Letzten_Tag_Im_Monat(Datum_2.Monat,Datum_2.Jahr);
      end if;
   else
      Datum_2.Jahr := Datum.Jahr;
      Datum_2.Monat := Datum.Monat;
      Datum_2.Tag := (Datum.Tag-1);
   end if;
   if Datum_2.Tag > Gib_Letzten_Tag_Im_Monat(Datum_2.Monat,Datum_2.Jahr) then
      raise Ungueltiges_Datum;
   end if;
   return Datum_2;
end Gib_Vorherigen_Tag;

-------------------------------------------------------------------------------
function Gib_Naechsten_Tag(Datum : Datum_Typ) return Datum_Typ is
-- Vorbedingung     : 1) Gueltiges Datum
--
-- Nachbedingung    : Liefert das Datum des vorherigen Tags. 
--
-- Fehlerbehandlung : 1) Ungueltiges Datum
-------------------------------------------------------------------------------
   Datum_2 : Datum_Typ;   
begin
   if (datum.Tag=Gib_Letzten_Tag_Im_Monat(Datum.Monat,Datum.Jahr)) then
      if (datum.Monat=Monat_Typ'Last ) then
	 if (datum.Jahr=Jahr_Typ'Last ) then
	    raise Ungueltiges_Datum;
	 else
	    datum_2.Jahr := (datum.Jahr+1);
	    datum_2.Monat := Monat_Typ'First;
	    datum_2.Tag := Tag_Typ'First;
	 end if;
      else
	 datum_2.Jahr := datum.Jahr;
	 datum_2.Monat :=(datum.Monat+1);
	 datum_2.Tag := Tag_Typ'First;
      end if;
   else
      datum_2.Jahr := datum.Jahr;
      datum_2.Monat := datum.Monat;
      datum_2.Tag :=(datum.Tag+1);
   end if;
   if Datum_2.Tag > Gib_Letzten_Tag_Im_Monat(Datum_2.Monat,Datum_2.Jahr) then
      raise Ungueltiges_Datum;
   end if;
   return Datum_2;
end Gib_Naechsten_Tag;



--===========================================================================--
begin -- adb --
  Aktuelles_Referenzjahr := 1996;
  Berechne_Feiertage(Aktuelles_Referenzjahr);
end kalender; -- kalender.adb --


