--*****************************************************************************
-- Datei            : kalender.ads
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 1
-- Autor            : Christoph Schemainda / Peter Vollmer
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 20.12.95
-------------------------------------------------------------------------------
-- Aufgabe          : Stellt Funktionen fuer den Umgang mit den Kalender-
--		      funktionen zur Verfuegung. 
-- Anwendung        :
-- Einschraenkung   : Es wird nur der Zeitraum vom 01.01.1990 bis 31.12.2100
--                    beruecksichtigt.
-- Fehlerbehandlung : Es werden die Exceptions Ungueltiges_Datum und
--                    Ungueltige_Wochennummer zurueckgegeben, falls die
--                    Parameter der Funktionen den gueltigen Bereich
--                    ueberschreiten.
--*****************************************************************************
with Text_IO;use Text_IO;

package kalender is

type Tag_Typ is new Integer range 1 .. 31;
type Monat_Typ is new Integer range 1 .. 12;
type Jahr_Typ is new Integer range 1990 .. 2100;
type Wochennummer_Typ is new Integer range 1..54;
type Nummer_Des_Tages_Im_Jahr_Typ is new Integer range 1..366;  
type Wochentag_Typ is 
                 (Montag,Dienstag,Mittwoch,Donnerstag,Freitag,Samstag,Sonntag);
type Wochentag_Nummer_Typ is new Integer range 1..7;
type Datum_Typ is record
  Tag   : Tag_Typ;
  Monat : Monat_Typ;
  Jahr  : Jahr_Typ;
end record;

Ungueltiges_Datum : exception;
Ungueltige_Wochennummer : exception;

-------------------------------------------------------------------------------
function Ist_Groesser(Datum1,Datum2 : Datum_Typ) return Boolean;
			  
-- Vorbedingung     : 1) Gueltiges Datum
--
-- Nachbedingung    : vergleich zweier Datum_typen 
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum"
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Ist_Schaltjahr 
  (Jahr : Jahr_Typ)
   return Boolean ;
-- Vorbedingung     : 1) Jahr ist gueltiges Jahr
--
-- Nachbedingung    : Liefert TRUE wenn Datum in einem Schaltjahr liegt sonst 
--                    FALSE.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-- Erklaerung       : Alle Jahre welche durch 4 ohne Rest teilbar sind ausser
--                    den Jahren welche durch 100 aber nicht durch 400 teilbar
--                    sind. 
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
function Ist_Feiertag 
  (Datum : Datum_Typ)
   return Boolean ;
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Ist Datum ein Feiertag wird TRUE zurueckgegeben,
--			 sonst FALSE.
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Feiertagname 
  (Datum : Datum_Typ)
   return String ;
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Ist Datum ein Feiertag wird der Name des Feiertags 
--                    zurueckgegeben, sonst ein leerer String .
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Datum_In_Wochentag 
  (Datum : Datum_Typ)
   return Wochentag_Typ; 
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Liefert den Wochentag des Datums zurueck.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Datum_In_Nummer_Des_Tages_Im_Jahr (Datum : Datum_Typ) return 
                                                  Nummer_Des_Tages_Im_Jahr_Typ;
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Liefert die Nummer Des Tages im Jahr.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum"
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Nummer_Des_Tages_Im_Jahr_In_Datum 
  (Nummer : Nummer_Des_Tages_Im_Jahr_Typ;
   Jahr   : Jahr_Typ)
   return Datum_Typ;
-- Vorbedingung     : -
--
-- Nachbedingung    : Liefert das Datum zur Nummer des Tages im Jahr.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Datum_In_Wochennummer 
  (Datum : Datum_Typ)
   return Wochennummer_Typ;
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum.
--
-- Nachbedingung    : Liefert zu einem Datum die zugehoerige Wochennummer.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Erster_Tag_Der_Woche 
  (Woche : Wochennummer_Typ;
  Jahr   : Jahr_Typ)
   return Datum_Typ;
-- Vorbedingung     : -
--
-- Nachbedingung    : Liefert zur Wochennummer den ersten Tag der Woche.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Datum_In_String 
  (Datum : Datum_Typ)
   return String;
-- Vorbedingung     : 1) Datum ist ein gueltiges Datum. 
--
-- Nachbedingung    : Wandelt Datum_Typ in String der Form TT.MM.JJJJ . 
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------
 
-------------------------------------------------------------------------------
function String_In_Datum 
  (Datumstring : String)
   return Datum_Typ;
-- Vorbedingung     : 1) Datumstring ist ein String der Form TT.MM.JJJJ .
--		     	
--
-- Nachbedingung    : Liefert das Datum aus einem String.
--
-- Fehlerbehandlung : 1) Exception "Ungueltiges_Datum".
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Gib_Letzten_Tag_Im_Monat
  (Monat : Monat_Typ;Jahr : Jahr_Typ)
   return Tag_Typ;
-- Vorbedingung     : -
--
-- Nachbedingung    : Liefert die Anzahl der Tage eines Monts.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Gib_Vorherigen_Tag(Datum : Datum_Typ) return Datum_Typ;
-- Vorbedingung     : 1) Gueltiges Datum
--
-- Nachbedingung    : Liefert das Datum des vorherigen Tags. 
--
-- Fehlerbehandlung : 1) Ungueltiges Datum
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function Gib_Naechsten_Tag(Datum : Datum_Typ) return Datum_Typ;
-- Vorbedingung     : 1) Gueltiges Datum
--
-- Nachbedingung    : Liefert das Datum des vorherigen Tags. 
--
-- Fehlerbehandlung : 1) Ungueltiges Datum
-------------------------------------------------------------------------------

end kalender;
