-- ****************************************************************************
-- Datei            : terminverwaltung.ads
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 1
-- Autor            : Christoph Schemainda
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 17.12.95
-------------------------------------------------------------------------------
-- Aufgabe          : Verwaltet alle Termine des Benutzers und stellt die 
--                    benoetigten Funktionen und Prozeduren fuer den Zugrif
--                    auf die Daten zur verfuegung.Ungueltige_Wochennummer 
--                    Auf die einzelnen Termine kann ueber das Datum und die
--                    Anfangszeit zugegriffen werden.
-- Anwendung        : Bevor auf die Termine zegegriffen werden kann, muessen
--                    die Termine aus der Datei gelesen werden.
-- Einschraenkung   : Bei den Terminen koennen keine Terminueberschneidungen 
--                    vorkommen.
-- Fehlerbehandlung : Die Exception "Kein_Termin" wird ausgeloest, wenn 
--                    versucht wird auf einen nicht existierenden Termin
--                    zuzugreifen.
--                    Die Exception "Terminueberschneidung" wird ausgeloest, 
--                    wenn es beim Termin eintragen zu Terminueberschneidung
--                    kommen wuerde.
--*****************************************************************************
with Text_Io;use Text_Io;
with Sequential_IO;
with Stringzuweisung;use Stringzuweisung;
with Uhrzeit;
package body terminverwaltung is

package     Termin_File_IO is new Sequential_IO (Termin_Typ);
package Teilnehmer_File_IO is new Sequential_IO (Teilnehmer_Liste_Typ);
Termin_Liste_Name     : constant String := "Termine.";
Teilnehmer_Liste_Name : constant String := "Teilnehmer.";

Termin_File     :     Termin_File_Io.File_Type;
Teilnehmer_File : Teilnehmer_File_Io.File_Type;
Termine_Im_Jahr : Termine_Der_Einzelnen_Jahre_Typ;
procedure Listenausgabe;
-------------------------------------------------------------------------------
function Termin_liegt_Spaeter(Termin1,Termin2 : Termin_Typ) return Boolean is
-- Vorbedingung     : 
--
-- Nachbedingung    : Liefert True wenn Termin1 groesser als Termin2 ist,sonst False.
--                    Achtung !!! Der Vergleich bezieht sich nicht auf das Jahr. 
--
-- Fehlerbehandlung : 
-------------------------------------------------------------------------------
Dummy : Uhrzeit.Dauer_Typ;
begin
   if Kalender."="(Termin1.Datum.Monat,Termin2.Datum.Monat) then 
      if Kalender."="(Termin1.Datum.Tag,Termin2.Datum.Tag) then
     Dblock :
	     begin
	       Dummy := Uhrzeit.Zeitdifferenz(Termin1.Anfangszeit,Termin2.Anfangszeit);
	     exception when Uhrzeit.Ungueltige_Dauer => return true;
	     end Dblock;
      return False;
     
      else 
	 if Kalender.">"(Termin1.Datum.Tag,Termin2.Datum.Tag) then return True;
	 else return False;
	 end if;
      end if;
   else
      if Kalender.">"(Termin1.Datum.Monat,Termin2.Datum.Monat) then return True;
      else return False;
      end if;
   end if;
end Termin_Liegt_spaeter;



-------------------------------------------------------------------------------
procedure Lese_Daten is
-- Vorbedingung     : -
--
-- Nachbedingung    : Alle Termine des Benutzers werden Eingelesen und die
--                    Datenstrukturen werden erzeugt.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

   Phelp : P_Termin_Liste_Typ;
   THelp : P_Teilnehmer_Liste_Typ;
begin
   for I in Kalender.Jahr_typ'Range loop
  Lese_Jahr :
      begin
	-- Beide Dateien oeffnen --
	Termin_File_Io.Open(Termin_File,Termin_File_Io.In_File,
	       Benutzer.Pfad&Termin_Liste_Name&Int_To_String(Integer(I)));
	Teilnehmer_File_Io.Open(Teilnehmer_File,Teilnehmer_File_Io.In_File,
	       Benutzer.Pfad&Teilnehmer_Liste_Name&Int_To_String(Integer(I)));
	Termine_Im_Jahr(I) := new Termin_Liste_Typ;
	PHelp := Termine_Im_Jahr(I);

	-- Einlesen des ersten Termins --
	Termin_File_Io.Read(Termin_File,PHelp.Termin);
	-- Einlesen der Teilnehmerliste des ersten Termins (wenn vorhanden) --
	if (PHelp.Termin.Teilnehmer_Liste /= null) then
	   PHelp.Termin.Teilnehmer_Liste := new Teilnehmer_Liste_Typ;
	   THelp := Phelp.Termin.Teilnehmer_Liste;
	   Teilnehmer_File_Io.Read(Teilnehmer_File,Thelp.all);	   
	   -- Einlesen der restlichen Teilnehmerliste --
	   while (THelp.Naechster /= null) loop
	      THelp.Naechster := new Teilnehmer_Liste_Typ;
	      Thelp := Thelp.Naechster;
	      Teilnehmer_File_Io.Read(Teilnehmer_File,THelp.all);
	   end loop;
	end if;
	-- Einlesen der restlichen Termine --
	while not Termin_File_Io.End_Of_File(Termin_File) loop
	   PHelp.Naechster := new Termin_Liste_Typ;
	   Phelp := PHelp.Naechster;
	   Termin_File_Io.Read(Termin_File,PHelp.Termin);
	   if (PHelp.Termin.Teilnehmer_Liste /= null) then
	      PHelp.Termin.Teilnehmer_Liste := new Teilnehmer_Liste_Typ;
	      THelp := Phelp.Termin.Teilnehmer_Liste;
	      Teilnehmer_File_Io.Read(Teilnehmer_File,Thelp.all);
	      -- Einlesen der restlichen Teilnehmerliste --
	      while (THelp.Naechster /= null) loop
		 THelp.Naechster := new Teilnehmer_Liste_Typ;
		 Thelp := Thelp.Naechster;
		 Teilnehmer_File_Io.Read(Teilnehmer_File,THelp.all);
	      end loop;
	   end if;
	end loop;
	   
	Termin_File_Io.Close(Termin_File);
	Teilnehmer_File_Io.Close(Teilnehmer_File);
      exception  
	 when Termin_File_Io.Name_Error => 
	    Termine_Im_Jahr(I) := null;
      when others => 
	 Put_Line("Fehler beim Lesen  :"&Kalender.Jahr_Typ'Image(I));raise;
      end Lese_Jahr;
  end loop;
   
   -- Termin_File_Io.Close(Termin_File);
   -- Teilnehmer_File_Io.Close(Teilnehmer_File);
end Lese_Daten;

-------------------------------------------------------------------------------
procedure Schreibe_Daten ( Jahr : in Kalender.Jahr_Typ ) is
-- Vorbedingung     : -
--
-- Nachbedingung    : Die Termine der angegebenen Jahres werden in eine Datei
--                    gespeichert, dabei wird eine eventuell schon vorhandene
--                    Datei ueberschrieben.
 --
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

  Phelp : P_Termin_Liste_Typ;
  THelp : P_Teilnehmer_Liste_Typ;
  I : Kalender.Jahr_Typ;
begin
    I:= Jahr;
    -- Falls keine Termine fuer das Jahr vorliegen , wird das File geloescht --
    -- ansosten die aktuelle Belegung abgelegt. --
      Schreibe_Jahr : 
	begin                            
	if (Termine_Im_Jahr(I) = null) then
	   Termin_File_Io.Open(Termin_File,Termin_File_Io.In_File,
			       Benutzer.Pfad&Termin_Liste_Name&
			       Int_To_String(Integer(I)));
	   Termin_File_Io.Delete(Termin_File);
	else 
	   Termin_File_Io.Create(Termin_File,Termin_File_Io.Out_File,
				 Benutzer.Pfad&Termin_Liste_Name&
				 Int_To_String(Integer(I)));
	   Teilnehmer_File_Io.Create(Teilnehmer_File,
				     Teilnehmer_File_Io.Out_File,
				     Benutzer.Pfad&Teilnehmer_Liste_Name&
				     Int_To_String(Integer(I)));
	   -- Termine schreiben --
	   PHelp := Termine_Im_Jahr(I);
	   while (PHelp /= null) loop
	      Termin_File_Io.Write(Termin_File,PHelp.Termin);
	      THelp := PHelp.Termin.Teilnehmer_Liste;
	      while (THelp/= null) loop
		 Teilnehmer_File_Io.Write(Teilnehmer_File,THelp.all);
		 THelp := THelp.Naechster;
	      end loop;
	      PHelp := PHelp.Naechster;
	   end loop;
	   Termin_File_Io.Close(Termin_File);
	    Teilnehmer_File_Io.Close(Teilnehmer_File);
	   
	end if;
	exception when Termin_File_Io.Name_Error => Put_Line("Fehler beim schreiben");
	end Schreibe_Jahr;
	
   
end;

-------------------------------------------------------------------------------
function Gib_Termin ( Anfangszeit : Uhrzeit.Uhrzeit_Typ;  
                      Datum       : Kalender.Datum_Typ ) 
                    return Termin_Typ is
-- Vorbedingung     : 1) Es existiert ein Termin am angegebenem Datum mit der
--                       angegebenen Anfangszeit.
--
-- Nachbedingung    : Es werden alle Daten des gesuchten Termins zurueckgegeben
--
-- Fehlerbehandlung : 1) Exception "Kein_Termin" wird ausgeloest.
-------------------------------------------------------------------------------

Hilfs : P_Termin_Liste_Typ;
begin
   
   Hilfs := Termine_Im_Jahr(Datum.Jahr);
   while Hilfs /= null loop
      if Kalender."="(Hilfs.Termin.Datum,Datum) and 
	Uhrzeit."="(Hilfs.Termin.Anfangszeit,Anfangszeit) then
	 return Hilfs.Termin;
      end if;
      Hilfs := Hilfs.Naechster;
   end loop;
   raise Kein_Termin; 
end;

-------------------------------------------------------------------------------
procedure Termin_Einfuegen( Platz : in out P_Termin_Liste_Typ;
			    Termin : in Termin_Typ) is
-- Vorbedingung     :
--
-- Nachbedingung    : Fuegt den Termin an den vorgesehenen platz ein. 
--
-- Fehlerbehandlung : 
-------------------------------------------------------------------------------
Alter_Pointer : P_Termin_Liste_Typ;
Teilnehmer,Neuer_Teilnehmer : P_Teilnehmer_Liste_Typ;

begin

Alter_Pointer:=Platz;
Platz := new Termin_Liste_Typ;
Platz.Termin := Termin;
Platz.Naechster := Alter_Pointer;

Neuer_Teilnehmer := Termin.Teilnehmer_Liste;
if Neuer_Teilnehmer/=null then 
   Platz.Termin.Teilnehmer_Liste := new Teilnehmer_Liste_Typ;
   Teilnehmer := Platz.Termin.Teilnehmer_Liste;
   Teilnehmer.Name := Neuer_Teilnehmer.Name;
   Neuer_Teilnehmer:=Neuer_Teilnehmer.Naechster;
   while Neuer_Teilnehmer /= null loop
      Teilnehmer.Naechster := new Teilnehmer_Liste_Typ;
      Teilnehmer := Teilnehmer.Naechster;
      Teilnehmer.Name := Neuer_Teilnehmer.Name;
      Neuer_Teilnehmer:=Neuer_Teilnehmer.Naechster;
   end loop;
else
   Platz.Termin.Teilnehmer_Liste := null;
end if;
end Termin_Einfuegen;


-------------------------------------------------------------------------------
procedure Termin_Eintragen ( Termin : in Termin_Typ ) is
-- Vorbedingung     : 1) Das Eintragen des Termins fuehrt zu keiner
--                       Terminueberschneidung.
--
-- Nachbedingung    : Der Termin wurde in die Datenstruktur Eingetragen.
--
-- Fehlerbehandlung : 1) Exception "Terminueberschneidung" wird ausgeloest.
-------------------------------------------------------------------------------
PHelp,JHelp,JVorher : P_Termin_Liste_Typ;

THelp,THelp2,T_In_Help : P_Teilnehmer_Liste_Typ;
Jahr : Kalender.Jahr_Typ;
Weiter_Suchen : Boolean;

begin
Weiter_Suchen := TRUE;
Jahr := Termin.Datum.Jahr; -- Variable nur wegen Uebersichtlichkeit 
    
if Termine_Im_Jahr(Jahr) = Null then
   Termin_Einfuegen(Termine_Im_Jahr(Jahr),Termin);
elsif Termin_Liegt_Spaeter(Termine_Im_Jahr(Jahr).Termin,Termin) then
   Termin_Einfuegen(Termine_Im_Jahr(Jahr),Termin);
else
   JVorher := Termine_Im_Jahr(Jahr);
   JHelp := Termine_Im_Jahr(Jahr).Naechster;
   while (Weiter_Suchen and JHelp /= null) loop
      if Termin_Liegt_Spaeter(Jhelp.Termin,Termin) then
	 Termin_Einfuegen(JVorher.Naechster,Termin);
	 Weiter_Suchen := FALSE ;
      else
	 Jhelp := JHelp.Naechster;
	 JVorher := JVorher.Naechster;
      end if;
   end loop;
   if Weiter_Suchen then
      Termin_Einfuegen(JVorher.Naechster,Termin);
   end if;
end if;
Schreibe_Daten(Jahr);
end Termin_Eintragen;

-------------------------------------------------------------------------------
procedure Termin_Loeschen ( Anfangszeit : in Uhrzeit.Uhrzeit_Typ ;
                            Datum       : in Kalender.Datum_Typ ) is
-- Vorbedingung     : 1) Es existiert ein Termin am angegebenem Datum mit der
--                       angegebenen Anfangszeit.
--
-- Nachbedingung    : Der angegebene Termin wird aus der Datenstruktur
--                    geloescht.
--
-- Fehlerbehandlung : 1) Exception "Kein_Termin" wird ausgeloest.
-------------------------------------------------------------------------------
Hilfs,Letzter : P_Termin_Liste_Typ;
begin
Hilfs := Termine_Im_Jahr(Datum.Jahr);
if (Kalender."="(Hilfs.Termin.Datum,Datum) and 
    Uhrzeit."="(Hilfs.Termin.Anfangszeit,Anfangszeit)) then
   Termine_Im_Jahr(Datum.Jahr):=Termine_Im_Jahr(Datum.Jahr).Naechster;
else
   Letzter := Hilfs;   
   Hilfs := Hilfs.Naechster;
   while Hilfs /= null loop
      if (Kalender."="(Hilfs.Termin.Datum,Datum) and 
	  Uhrzeit."="(Hilfs.Termin.Anfangszeit,Anfangszeit)) then
	 Letzter.Naechster:=Hilfs.Naechster;
      end if;
      Letzter := Hilfs;   
      Hilfs := Hilfs.Naechster;
   end loop;
end if;
Schreibe_Daten(Datum.Jahr);

end;
-------------------------------------------------------------------------------
function Ist_Termin_Moeglich ( Termin : in Termin_Typ ) return Boolean is
-- Vorbedingung     : -
--
-- Nachbedingung    : Falls der angegebene Termin ohne Terminueberschneidung
--                    eingetragen werden koennte, liefert die Funktion TRUE
--                    ansonsten liefert sie FALSE.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------
T : P_Termin_Liste_Typ;
A1,A2,E1,E2 : Uhrzeit.Uhrzeit_Typ;
begin
   A1 := Termin.Anfangszeit;
   E1 := Termin.Endzeit;
 
   T := Termine_Im_Jahr(Termin.Datum.Jahr);
   while T /= null loop
      if Kalender."="(T.Termin.Datum,Termin.Datum) then
	 A2 := T.Termin.Anfangszeit;
	 E2 := T.Termin.Endzeit;
	 if Uhrzeit.Ist_kleiner(A1,A2) then 
	    if Uhrzeit.Ist_Kleiner(A2,E1) then return False;
	    end if;
	 elsif Uhrzeit.Ist_Kleiner(A1,E2) then return False;
	 end if;
      end if;
      T := T.Naechster;
   end loop;
   return True;
end;


-------------------------------------------------------------------------------
function Gib_Termine_Des_Tages ( Datum : in Kalender.Datum_Typ )
                               return P_Termin_Liste_Typ is
-- Vorbedingung     : -
--
-- Nachbedingung    : Es wird eine Liste mit allen Terminen eines Tages
--                    zurueckgegeben.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------
Liste,Hilfs,Alle_termine : P_Termin_Liste_Typ ;

Termin_Vorhanden : Boolean;

begin
Termin_Vorhanden := FALSE;
Alle_Termine := Termine_Im_Jahr(Datum.Jahr);

while Alle_Termine /= null loop
   if Kalender."="(Alle_Termine.Termin.Datum,Datum) then
      if Termin_Vorhanden then
	 Termin_Einfuegen(Hilfs.Naechster,Alle_Termine.Termin);
	 Hilfs := Hilfs.Naechster;
      else
	 Termin_Einfuegen(Liste,Alle_Termine.Termin);
	 Hilfs := Liste;
	 Termin_Vorhanden := TRUE;
      end if;
   end if;
   Alle_Termine := Alle_Termine.Naechster;
end loop;


return Liste;
   
end;

-------------------------------------------------------------------------
function Ist_Termin_Am_Tag ( Datum : in Kalender.Datum_Typ ) return Boolean is
-- Vorbedingung     : -
--
-- Nachbedingung    : Wenn Termine am gegebenem Tag existieren wird TRUE
--                    zurueckgegeben, ansonsten FALSE.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------
HListe : P_Termin_Liste_Typ; 
begin
 if Termine_Im_Jahr(Datum.Jahr) = null then return False;
 end if;
 HListe := Termine_Im_Jahr(Datum.Jahr);
 while HListe /= null loop
    if Kalender."="(HListe.Termin.Datum,Datum) then return True;
    end if;
    HListe:=  Hliste.Naechster;
 end loop;
 return False;
end;


-------------------------------------------------------------------------------
procedure Listenausgabe is
-- Vorbedingung     : -
--
-- Nachbedingung    : Testprozedur
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------
T : P_Termin_Liste_Typ;
begin


   T := Termine_Im_Jahr(1996);
   while T /= null loop
      Put_Line("Termin am   : "&Kalender.Datum_In_String(T.Termin.Datum));
      Put_Line("Anfangszeit : "&Uhrzeit.Zeit_In_String(T.Termin.Anfangszeit));
      Put_Line("Kurzbeschr. : "&String(T.Termin.Kurzbeschreibung));
      New_Line;
      T := T.Naechster;
   end loop;

end Listenausgabe;

--============================================================================
begin
null;
end terminverwaltung;








