--*****************************************************************************
-- Datei            : terminverwaltung.ads
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 1
-- Autor            : Christoph Schemainda
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 17.12.95
-------------------------------------------------------------------------------
-- Aufgabe          : Verwaltet alle Termine des Benutzers und stellt die 
--                    benoetigten Funktionen und Prozeduren fuer den Zugrif
--                    auf die Daten zur verfuegung.Ungueltige_Wochennummer 
--                    Auf die einzelnen Termine kann ueber das Datum und die
--                    Anfangszeit zugegriffen werden.
-- Anwendung        : Bevor auf die Termine zegegriffen werden kann, muessen
--                    die Termine aus der Datei gelesen werden.
-- Einschraenkung   : Bei den Terminen koennen keine Terminueberschneidungen 
--                    vorkommen.
-- Fehlerbehandlung : Die Exception "Kein_Termin" wird ausgeloest, wenn 
--                    versucht wird auf einen nicht existierenden Termin
--                    zuzugreifen.
--                    Die Exception "Terminueberschneidung" wird ausgeloest, 
--                    wenn es beim Termin eintragen zu Terminueberschneidung
--                    kommen wuerde.
--*****************************************************************************

with uhrzeit;
with kalender;
with Benutzer;

package terminverwaltung is
Pfad : constant String := "../daten/";
type Kurzbeschreibung_Typ is new String (1 .. 50);
type Ort_Typ is new String (1 .. 50);
type Name_Typ is new String (1 .. 50);
type Kommentar_Typ is array(1 .. 3) of String (1 .. 50);

type Teilnehmer_Liste_Typ;
type P_Teilnehmer_Liste_Typ is access Teilnehmer_Liste_Typ;
type Teilnehmer_Liste_Typ is record
  Name      : Name_Typ;
  Naechster : P_Teilnehmer_Liste_Typ;
end record;

type Termin_Typ is record
  Datum            : Kalender.Datum_Typ;
  Anfangszeit      : Uhrzeit.Uhrzeit_Typ;
  Endzeit          : Uhrzeit.Uhrzeit_Typ;
  Kurzbeschreibung : Kurzbeschreibung_Typ;
  Kommentar        : Kommentar_Typ;
  Ort              : Ort_Typ;
  Teilnehmer_Liste : P_Teilnehmer_Liste_Typ;
  Ist_Periodisch   : Boolean;
end record;

type Termin_Liste_Typ;
type P_Termin_Liste_Typ is access Termin_Liste_Typ;
type Termin_Liste_Typ is record
  Termin    : Termin_Typ;
  Naechster : P_Termin_Liste_Typ;
end record;

type Termine_Der_Einzelnen_Jahre_Typ  is array(Kalender.Jahr_Typ'range) of P_Termin_Liste_Typ; 

Kein_Termin : exception;
Terminueberschneidung : exception;

-------------------------------------------------------------------------------
procedure Lese_Daten ;
-- Vorbedingung     : -
--
-- Nachbedingung    : Alle Termine des Benutzers werden Eingelesen und die
--                    Datenstrukturen werden erzeugt.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
procedure Schreibe_Daten ( Jahr : in Kalender.Jahr_Typ );
-- Vorbedingung     : -
--
-- Nachbedingung    : Die Termine der angegebenen Jahres werden in eine Datei
--                    gespeichert, dabei wird eine eventuell schon vorhandene
--                    Datei ueberschrieben.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Gib_Termin ( Anfangszeit : Uhrzeit.Uhrzeit_Typ ;
                      Datum       : Kalender.Datum_Typ )
                    return Termin_Typ ;
-- Vorbedingung     : 1) Es existiert ein Termin am angegebenem Datum mit der
--                       angegebenen Anfangszeit.
--
-- Nachbedingung    : Es werden alle Daten des gesuchten Termins zurueckgegeben
--
-- Fehlerbehandlung : 1) Exception "Kein_Termin" wird ausgeloest.
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
procedure Termin_Eintragen ( Termin : in Termin_Typ ) ;
-- Vorbedingung     : 1) Das Eintragen des Termins fuehrt zu keiner
--                       Terminueberschneidung.
--
-- Nachbedingung    : Der Termin wurde in die Datenstruktur Eingetragen.
--
-- Fehlerbehandlung : 1) Exception "Terminueberschneidung" wird ausgeloest.
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
procedure Termin_Loeschen ( Anfangszeit : in Uhrzeit.Uhrzeit_Typ ;
                            Datum       : in Kalender.Datum_Typ );
-- Vorbedingung     : 1) Es existiert ein Termin am angegebenem Datum mit der
--                       angegebenen Anfangszeit.
--
-- Nachbedingung    : Der angegebene Termin wird aus der Datenstruktur
--                    geloescht.
--
-- Fehlerbehandlung : 1) Exception "Kein_Termin" wird ausgeloest.
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Ist_Termin_Moeglich ( Termin : in Termin_Typ ) return boolean;
-- Vorbedingung     : -
--
-- Nachbedingung    : Falls der angegebene Termin ohne Terminueberschneidung
--                    eingetragen werden koennte, liefert die Funktion TRUE
--                    ansonsten liefert sie FALSE.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Gib_Termine_Des_Tages ( Datum : in Kalender.Datum_Typ )
                               return P_Termin_Liste_Typ;
-- Vorbedingung     : -
--
-- Nachbedingung    : Es wird eine Liste mit allen Terminen eines Tages
--                    zurueckgegeben.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Ist_Termin_Am_Tag ( Datum : in Kalender.Datum_Typ ) return Boolean ;
-- Vorbedingung     : -
--
-- Nachbedingung    : Wenn Termine am gegebenem Tag existieren wird TRUE
--                    zurueckgegeben, ansonsten FALSE.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

end terminverwaltung;
