--*****************************************************************************
-- Datei            : uhrzeit.adb
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 1.0
-- Autor            : Christoph Schemainda
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 10.12.95
-------------------------------------------------------------------------------
-- Aufgabe          : Stellt Funktionen fuer den Umgang mit Uhrzeiten zur
--                    Verfuegung
-- Anwendung        :
-- Einschraenkung   :
-- Fehlerbehandlung : Die Exception "Ungueltige_Uhrzeit" wird ausgeloest, wenn
--                    an ein Unterprogramm eine falsche Uhrzeit uebergeben
--                    wird.
--                    Die Exception "Ungueltige_Dauer" wird ausgeloest, wenn
--                    an ein Unterprogramm eine falsche Dauer uebergeben wird,
--                    oder wenn eine Berechnung einen negativen Wert liefern
--                    wuerde.
--*****************************************************************************
with Text_Io;use Text_Io;
package body uhrzeit is


-------------------------------------------------------------------------------
function Zeitdifferenz (von : Uhrzeit_Typ ;
                        bis : Uhrzeit_Typ ) return Dauer_Typ is
-- Vorbedingung     : 1) Von ist kleiner oder gleich bis.
--
-- Nachbedingung    : Die Dauer zwischen von und bis wird als Ergebnis 
--                    zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Dauer"
-------------------------------------------------------------------------------
Dauer : Dauer_Typ;

begin
Dauer.Stunden:=Bis.Stunden-Von.Stunden;
Dauer.Minuten:=Bis.Minuten-Von.Minuten;
if Dauer.Minuten<0 then
   Dauer.Minuten:=Dauer.Minuten+60;
   Dauer.Stunden:=Dauer.Stunden-1;
end if;

if Dauer.Stunden<0 then
   raise Ungueltige_Dauer ;
end if;
return Dauer;
exception
   when Ungueltige_Dauer =>
      raise;
   when others =>
      Text_Io.Put_Line("OOPS! Fehler in Uhrzeit.Zeitdifferenz!");
      raise;
end Zeitdifferenz;

-------------------------------------------------------------------------------
function Ist_kleiner (Zeit1,Zeit2 : Uhrzeit_Typ) return Boolean is
-- Vorbedingung     : 1) Von ist kleiner oder gleich bis.
--
-- Nachbedingung    : Die Dauer zwischen von und bis wird als Ergebnis 
--                    zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Dauer"
-------------------------------------------------------------------------------

begin 
if Zeit1.Stunden < Zeit2.Stunden then
   return TRUE;
elsif (Zeit1.Stunden=Zeit2.Stunden)and(Zeit1.Minuten < Zeit2.Minuten) then
   return TRUE;
else
   return FALSE ;
end if;
end Ist_Kleiner;

-------------------------------------------------------------------------------
function Addiere_Dauer (Zeit  : Uhrzeit_Typ;
                        Dauer : Dauer_Typ ) return Uhrzeit_Typ is
-- Vorbedingung     : 1) Die neue Uhrzeit darf nicht ueber die 24:00 Uhr
--                       Grenze reichen.
--
-- Nachbedingung    : Die Dauer wird auf die Uhrzeit addiert und das Ergebnis
--                    wird zurueckgegeben.
--
-- Fehlerbehandlung : Exception "Ungueltige_Uhrzeit"
-------------------------------------------------------------------------------
Zeit2 : Uhrzeit_Typ;

begin
Zeit2.Stunden:=Zeit.Stunden+Dauer.Stunden;
Zeit2.Minuten:=Zeit.Minuten+Dauer.Minuten;
if Zeit2.Minuten>=60 then
   Zeit2.Minuten:=Zeit2.Minuten-60;
   Zeit2.Stunden:=Zeit2.Stunden+1;
end if;
if Zeit.Stunden>=24 then
   raise Ungueltige_Uhrzeit;
end if;
return Zeit2;
exception
   when Ungueltige_Uhrzeit =>
      raise;
   when others =>
      Text_Io.Put_Line("OOPS! Fehler in Uhzeit.Addiere_Dauer!");
      raise;
end Addiere_Dauer;



-------------------------------------------------------------------------------
function Zeit_In_String (Zeit : Uhrzeit_Typ ) return String is
-- Vorbedingung     : -
--
-- Nachbedingung    : Die Uhrzeit wird als String der Form HH:MM zurueckgegeben.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------
ZeitString:String(1..5);

begin
ZeitString:="00:00";
if Zeit.Stunden < 10 then
   ZeitString(2..2):=Integer'Image(Zeit.Stunden)(2..2);
else
   ZeitString(1..2):=Integer'Image(Zeit.Stunden)(2..3);
end if;
if Zeit.Minuten < 10 then
   ZeitString(5..5):=Integer'Image(Zeit.Minuten)(2..2);
else
   ZeitString(4..5):=Integer'Image(Zeit.Minuten)(2..3);
end if;

return ZeitString;

exception
   when others =>
      Text_Io.Put_Line("OOPS! Fehler in Uhrzeit.Zeit_in_String!");
      raise;
end Zeit_In_String;



-------------------------------------------------------------------------------
function String_In_Zeit (ZeitString : String ) return Uhrzeit_Typ is
-- Vorbedingung     : 1) Der String muss eine gueltige Uhrzeit in der Form
--                       HH:MM enthalten.
--
-- Nachbedingung    : Es wird die entsprechende Uhrzeit zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Uhrzeit"
-------------------------------------------------------------------------------
Zeit : Uhrzeit_Typ;
Stunden,Minuten : String(1..2);

begin
Zeit.Stunden:=Integer'Value(ZeitString(1..2));
Zeit.Minuten:=Integer'Value(ZeitString(4..5));
if Zeit.Stunden>23 then raise Ungueltige_Uhrzeit; end if;
return Zeit;
exception
   when Constraint_Error => raise Ungueltige_Uhrzeit;
   when others =>
      Text_Io.Put_Line("OOPS! Fehler in Uhrzeit.Sting_In_Zeit!");
      raise;
end String_In_Zeit;

-------------------------------------------------------------------------------
function Dauer_In_String (Dauer : Dauer_Typ ) return String is
-- Vorbedingung     : -
--
-- Nachbedingung    : Die Dauer wird als String der Form HH:MM zurueckgegeben.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------
DauerString:String(1..5);


begin
DauerString:="00:00";
if Dauer.Stunden < 10 then
   DauerString(2..2):=Integer'Image(Dauer.Stunden)(2..2);
else
   DauerString(1..2):=Integer'Image(Dauer.Stunden)(2..3);
end if;
if Dauer.Minuten < 10 then
   DauerString(5..5):=Integer'Image(Dauer.Minuten)(2..2);
else
   DauerString(4..5):=Integer'Image(Dauer.Minuten)(2..3);
end if;
return DauerString;
exception
   when others =>
      Text_Io.Put_Line("OOPS! Fehler in Uhrzeit.Dauer_in_String!");
      raise;
end Dauer_In_String;

-------------------------------------------------------------------------------
function String_In_Dauer (DauerString : String ) return Dauer_Typ is
-- Vorbedingung     : 1) Der String muss eine gueltige Dauer in der Form
--                       HH:MM enthalten.
--
-- Nachbedingung    : Es wird die entsprechende Dauer zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Dauer"
-------------------------------------------------------------------------------
Dauer : Dauer_Typ;

begin
Dauer.Stunden:=Integer'Value(DauerString(1..2));
Dauer.Minuten:=Integer'Value(DauerString(4..5));
return Dauer ;
exception
   when others =>
      Text_Io.Put_Line("OOPS! Fehler in Uhrzeit.String_In_Dauer!");
      raise;
end String_In_Dauer;

end uhrzeit;

