--*****************************************************************************
-- Datei            : uhrzeit.ads
-- Project          : Terminkalender  ( FaPra WS1995/96 Gruppe 5 )
-- Version          : 1.0
-- Autor            : Christoph Schemainda
-- Hardware         : DEC-Workstation 3100
-- Betriebssystem   : Ultrix Version 4.3
-- Sprache          : Ada95
-- Compiler         : gnat Version 2.07
-- Erstellungsdatum : 10.12.95
-------------------------------------------------------------------------------
-- Aufgabe          : Stellt Funktionen fuer den Umgang mit Uhrzeiten zur
--                    Verfuegung
-- Anwendung        :
-- Einschraenkung   :
-- Fehlerbehandlung : Die Exception "Ungueltige_Uhrzeit" wird ausgeloest, wenn
--                    an ein Unterprogramm eine falsche Uhrzeit uebergeben
--                    wird.
--                    Die Exception "Ungueltige_Dauer" wird ausgeloest, wenn
--                    an ein Unterprogramm eine falsche Dauer uebergeben wird,
--                    oder wenn eine Berechnung einen negativen Wert liefern
--                    wuerde.
--*****************************************************************************

package uhrzeit is

type Uhrzeit_Typ is record 
   Stunden : Integer;
   Minuten : Integer;
end record;
type Dauer_Typ is new Uhrzeit_Typ;

Ungueltige_Dauer : exception;
Ungueltige_Uhrzeit : exception;

-------------------------------------------------------------------------------
function Zeitdifferenz (von : Uhrzeit_Typ;
                        bis : Uhrzeit_Typ ) return Dauer_Typ;
-- Vorbedingung     : 1) Von ist kleiner oder gleich bis.
--
-- Nachbedingung    : Die Dauer zwischen von und bis wird als Ergebnis 
--                    zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Dauer"
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function Ist_kleiner (Zeit1,Zeit2 : Uhrzeit_Typ) return Boolean;
-- Vorbedingung     : 1) Von ist kleiner oder gleich bis.
--
-- Nachbedingung    : Die Dauer zwischen von und bis wird als Ergebnis 
--                    zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Dauer"
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Addiere_Dauer (Zeit  : Uhrzeit_Typ;
                        Dauer : Dauer_Typ ) return Uhrzeit_Typ ;
-- Vorbedingung     : 1) Die neue Uhrzeit darf nicht ueber die 24:00 Uhr
--                       Grenze reichen.
--
-- Nachbedingung    : Die Dauer wird auf die Uhrzeit addiert und das Ergebnis
--                    wird zurueckgegeben.
--
-- Fehlerbehandlung : Exception "Ungueltige_Uhrzeit"
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Zeit_In_String (Zeit : Uhrzeit_Typ ) return String;
-- Vorbedingung     : -
--
-- Nachbedingung    : Die Uhrzeit wird als String der Form HH:MM zurueckgegeben.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function String_In_Zeit (ZeitString : String ) return Uhrzeit_Typ;
-- Vorbedingung     : 1) Der String muss eine gueltige Uhrzeit in der Form
--                       HH:MM enthalten.
--
-- Nachbedingung    : Es wird die entsprechende Uhrzeit zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Uhrzeit"
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function Dauer_In_String (Dauer : Dauer_Typ ) return String ;
-- Vorbedingung     : -
--
-- Nachbedingung    : Die Dauer wird als String der Form HH:MM zurueckgegeben.
--
-- Fehlerbehandlung : -
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
function String_In_Dauer (DauerString : String ) return Dauer_Typ;
-- Vorbedingung     : 1) Der String muss eine gueltige Dauer in der Form
--                       HH:MM enthalten.
--
-- Nachbedingung    : Es wird die entsprechende Dauer zurueckgegeben.
--
-- Fehlerbehandlung : 1) Exception "Ungueltige_Dauer"
-------------------------------------------------------------------------------

end uhrzeit;
